@@ -19,6 +19,26 @@
 
 #include "dirwalk.h"
 #include "progressbar.h"
+
+/*
+	The following section renames global variables defined in SDL_Pango.h to avoid errors during linking.
+	It is okay to rename these variables because they are constants.
+	SDL_Pang.h is included by fonts.h.  
+*/
+#define _MATRIX_WHITE_BACK _MATRIX_WHITE_BACK1
+#define MATRIX_WHITE_BACK MATRIX_WHITE_BACK1
+#define _MATRIX_BLACK_BACK _MATRIX_BLACK_BACK1
+#define MATRIX_BLACK_BACK MATRIX_BLACK_BACK1
+#define _MATRIX_TRANSPARENT_BACK_BLACK_LETTER _MATRIX_TRANSPARENT_BACK_BLACK_LETTER1
+#define MATRIX_TRANSPARENT_BACK_BLACK_LETTER MATRIX_TRANSPARENT_BACK_BLACK_LETTER1
+#define _MATRIX_TRANSPARENT_BACK_WHITE_LETTER _MATRIX_TRANSPARENT_BACK_WHITE_LETTER1
+#define MATRIX_TRANSPARENT_BACK_WHITE_LETTER MATRIX_TRANSPARENT_BACK_WHITE_LETTER1
+#define _MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER _MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER1
+#define MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER1
+/*
+	The renaming ends here.
+*/
+
 #include "fonts.h"
 
 #include "debug.h"
@@ -65,37 +85,43 @@ void loadfont_callback(SDL_Surface * screen, const char *restrict const dir,
     if (loadable)
     {
       char fname[512];
-      TTF_Font *font;
+      TuxPaint_Font *font;
       snprintf(fname, sizeof fname, "%s/%s", dir, files[i].str);
 //printf("Loading font: %s\n", fname);
-      font = TTF_OpenFont(fname, text_sizes[text_size]);
+      font = TuxPaint_Font_OpenFont("", fname, text_sizes[text_size]);
       if (font)
       {
-	const char *restrict const family = TTF_FontFaceFamilyName(font);
-	const char *restrict const style = TTF_FontFaceStyleName(font);
+	const char *restrict const family = TuxPaint_Font_FontFaceFamilyName(font);
+	const char *restrict const style = TuxPaint_Font_FontFaceStyleName(font);
 
 
 #ifdef DEBUG
-	int numfaces = TTF_FontFaces(font);
+	int numfaces = TTF_FontFaces(font->ttf_font);
 	if (numfaces != 1)
 	  printf("Found %d faces in %s, %s, %s\n", numfaces, files[i].str,
 		 family, style);
+
+        printf("success: tpf: 0x%x tpf->ttf_font: 0x%x\n",
+	       (unsigned int) font, (unsigned int) font->ttf_font);
 #endif
 
 	// First, the blacklist. We list font families that can crash Tux Paint
 	// via bugs in the SDL_ttf library. We also test fonts to be sure that
 	// they have both uppercase and lowercase letters. Note that we do not
 	// test for "Aa", because it is OK if uppercase and lowercase are the
-	// same. (but not nice -- such fonts get a low score later)
+	// same (but not nice -- such fonts get a low score later).
 	//
 	// We test the alphabet twice, to help with translation. If the users
-	// will be unable to type ASCII letters, then both lines should be
-	// translated. Otherwise, only Line X should be translated and the
-	// ASCII-only fonts should be given bad scores in the scoring code below.
-	// (the best scores going to fonts that support both)
+	// will be unable to type ASCII letters, then both Line X and Line Y
+	// should be translated. Otherwise, only Line X should be translated
+	// and the ASCII-only fonts should be given bad scores in the scoring
+	// code below (the best scores going to fonts that support both).
 	
-	if (strcmp("Zapfino", family) && strcmp("Elvish Ring NFI", family) && ((charset_works(font, gettext("qx")) && charset_works(font, gettext("QX")))	// Line X
-									       || (charset_works(font, gettext("qy")) && charset_works(font, gettext("QY")))	// Line Y
+	// Line X
+	if (strcmp("Zapfino", family) && strcmp("Elvish Ring NFI", family) && ((charset_works(font, gettext("qx")) && charset_works(font, gettext("QX")))
+
+	// Line Y
+									       || (charset_works(font, gettext("qy")) && charset_works(font, gettext("QY")))
 	    ))
 	{
 	  if (num_font_styles == num_font_styles_max)
@@ -111,19 +137,34 @@ void loadfont_callback(SDL_Surface * screen, const char *restrict const dir,
 	  user_font_styles[num_font_styles]->filename = files[i].str;	// steal it (mark NULL below)
 	  user_font_styles[num_font_styles]->family = strdup(family);
 	  user_font_styles[num_font_styles]->style = strdup(style);
+
 	  // Now we score fonts to ensure that the best ones will be placed at
 	  // the top of the list. The user will see them first. This sorting is
 	  // especially important for users who have scroll buttons disabled.
 	  // Translators should do whatever is needed to put crummy fonts last.
-	  user_font_styles[num_font_styles]->score = charset_works(font, gettext("oO"));	// distinct uppercase and lowercase
-	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("`\%_@$~#{}<>^&*"));	// uncommon punctuation
-	  user_font_styles[num_font_styles]->score += charset_works(font, gettext(",.?!"));	// common punctuation
-	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("017"));	// digits
-	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("O0"));	// distinct circle-like characters
-	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("1Il|"));	// distinct line-like characters
+	  
+	  // distinct uppercase and lowercase (e.g., 'o' vs. 'O')
+	  user_font_styles[num_font_styles]->score = charset_works(font, gettext("oO"));
+
+	  // uncommon punctuation (e.g., '@', '#', '*', etc.)
+	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("`\%_@$~#{}<>^&*"));
+
+	  // common punctuation (e.g., '?', '!', '.', ',', etc.)
+	  user_font_styles[num_font_styles]->score += charset_works(font, gettext(",.?!"));
+
+	  // digits (e.g., '0', '1' and '7')
+	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("017"));
+
+	  // distinct circle-like characters (e.g., 'O' (capital oh) vs. '0' (zero))
+	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("O0"));
+
+	  // distinct line-like characters (e.g., 'l' (lowercase elle) vs. '1' (one) vs. 'I' (capital aye))
+	  user_font_styles[num_font_styles]->score += charset_works(font, gettext("1Il|"));
+
 #ifdef OLPC_XO
 	  user_font_styles[num_font_styles]->score += charset_works(font, "\xc3\x97\xc3\xb7");	// multiply and divide
 #endif
+
 	  num_font_styles++;
 //printf("Accepted: %s, %s, %s, score(%d)\n", files[i].str, family, style, user_font_styles[num_font_styles]->score);
 	  files[i].str = NULL;	// so free() won't crash -- we stole the memory
@@ -136,7 +177,7 @@ void loadfont_callback(SDL_Surface * screen, const char *restrict const dir,
 		 style);
 #endif
 	}
-	TTF_CloseFont(font);
+	TuxPaint_Font_CloseFont(font);
       }
       else
       {
