@@ -17,6 +17,25 @@
 #define gettext_noop(String) String
 #endif
 
+/*
+	The following section renames global variables defined in SDL_Pango.h to avoid errors during linking.
+	It is okay to rename these variables because they are constants.
+	SDL_Pang.h is included by fonts.h.  
+*/
+#define _MATRIX_WHITE_BACK _MATRIX_WHITE_BACK2
+#define MATRIX_WHITE_BACK MATRIX_WHITE_BACK2
+#define _MATRIX_BLACK_BACK _MATRIX_BLACK_BACK2
+#define MATRIX_BLACK_BACK MATRIX_BLACK_BACK2
+#define _MATRIX_TRANSPARENT_BACK_BLACK_LETTER _MATRIX_TRANSPARENT_BACK_BLACK_LETTER2
+#define MATRIX_TRANSPARENT_BACK_BLACK_LETTER MATRIX_TRANSPARENT_BACK_BLACK_LETTER2
+#define _MATRIX_TRANSPARENT_BACK_WHITE_LETTER _MATRIX_TRANSPARENT_BACK_WHITE_LETTER2
+#define MATRIX_TRANSPARENT_BACK_WHITE_LETTER MATRIX_TRANSPARENT_BACK_WHITE_LETTER2
+#define _MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER _MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER2
+#define MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER MATRIX_TRANSPARENT_BACK_TRANSPARENT_LETTER2
+/*
+	The renaming ends here.
+*/
+
 #include "fonts.h"
 #include "i18n.h"
 #include "progressbar.h"
@@ -31,6 +50,12 @@
 #ifdef __APPLE__
 #include "wrapperdata.h"
 extern WrapperData macosx;
+
+/* system fonts that cause TTF_OpenFont to crash */
+static const char* problemFonts[] = {
+  "/Library/Fonts//AppleMyungjo.ttf", 
+  NULL 
+};
 #endif
 
 #ifdef FORKED_FONTS
@@ -59,13 +84,28 @@ extern WrapperData macosx;
 #ifndef FORKED_FONTS
 SDL_Thread *font_thread;
 #endif
+
+#ifndef NO_SDLPANGO
+
+#include "SDL_Pango.h"
+#if !defined(SDL_PANGO_H)
+#error "---------------------------------------------------"
+#error "If you installed SDL_Pango from a package, be sure"
+#error "to get the development package, as well!"
+#error "(e.g., 'libsdl-pango1-dev.rpm')"
+#error "---------------------------------------------------"
+#endif
+
+#endif
+
+
 int no_system_fonts;
 volatile long font_thread_done = 0, font_thread_aborted = 0;
 volatile long waiting_for_fonts = 0;
 int font_scanner_pid;
 int font_socket_fd;
 
-TTF_Font *medium_font, *small_font, *large_font, *locale_font;
+TuxPaint_Font *medium_font, *small_font, *large_font, *locale_font;
 
 family_info **user_font_families;
 int num_font_families = 0;
@@ -75,9 +115,10 @@ style_info **user_font_styles;
 int num_font_styles = 0;
 int num_font_styles_max = 0;
 
-int text_state;
+int text_state = 0;
 unsigned text_size = 4;		// initial text size
 
+
 /* Unfortunately, there is a bug in SDL_ttf-2.0.6, the current version
    that causes a segmentation fault if an attempt is made to call
    TTF_OpenFont() with the filename of a font that doesn't exist. This
@@ -108,8 +149,16 @@ TTF_Font *BUGFIX_TTF_OpenFont206(const char *const file, int ptsize)
 
 /* #define TTF_OpenFont    BUGFIX_TTF_OpenFont206 */
 
-TTF_Font *try_alternate_font(int size)
+TuxPaint_Font *try_alternate_font(int size)
 {
+#ifndef NO_SDLPANGO
+
+  /* Do we need to do anything when we use SDL_Pango?
+     -bjk 2007.07.12 */
+
+  (void)size;  // silence 'unused parameter' warning
+
+#else
   char str[128];
   char prefix[64];
   char *p;
@@ -120,39 +169,50 @@ TTF_Font *try_alternate_font(int size)
     *p = 0;
     snprintf(str, sizeof(str), "%sfonts/locale/%s.ttf", DATA_PREFIX, prefix);
 
-    return TTF_OpenFont(str, size);
+    return TuxPaint_Font_OpenFont("", str, size);
   }
+#endif
+
   return NULL;
 }
 
-TTF_Font *load_locale_font(TTF_Font * fallback, int size)
+TuxPaint_Font *load_locale_font(TuxPaint_Font * fallback, int size)
 {
-  TTF_Font *ret = NULL;
+  TuxPaint_Font *ret = NULL;
   if (need_own_font)
   {
+#ifndef NO_SDLPANGO
+
+    /* FIXME: Do we even need to do anything when using SDL_Pango? -bjk 2007.07.12 */
+
+    (void)size;  // silence 'unused parameter' warning
+
+    return(fallback);
+
+#else
     char str[128];
     snprintf(str, sizeof(str), "%sfonts/locale/%s.ttf",
 	     DATA_PREFIX, lang_prefix);
 
-    ret = TTF_OpenFont(str, size);
+    ret = TuxPaint_Font_OpenFont("", str, size);
 	 
 #ifdef __APPLE__
     if (ret == NULL)
     {
       snprintf(str, sizeof(str), "%sfonts/%s.ttf", DATA_PREFIX, lang_prefix);
-      ret = TTF_OpenFont(str, size);
+      ret = TuxPaint_Font_OpenFont("", str, size);
     }	 
 	 
     if (ret == NULL)
     {
       snprintf(str, sizeof(str), "/Library/Fonts/%s.ttf", lang_prefix);
-      ret = TTF_OpenFont(str, size);
+      ret = TuxPaint_Font_OpenFont("", str, size);
     }
 	 
     if (ret == NULL)
     {
       snprintf(str, sizeof(str), "%s/%s.ttf", macosx.fontsPath, lang_prefix);
-      ret = TTF_OpenFont(str, size);
+      ret = TuxPaint_Font_OpenFont("", str, size);
     }	 
 #endif
 
@@ -180,10 +240,116 @@ TTF_Font *load_locale_font(TTF_Font * fallback, int size)
 	set_current_language();
       }
     }
+
+#endif
   }
   return ret ? ret : fallback;
 }
 
+void TuxPaint_Font_CloseFont(TuxPaint_Font * tpf)
+{
+#ifndef NO_SDLPANGO
+  if (tpf->typ == FONT_TYPE_PANGO)
+  {
+    SDLPango_FreeContext(tpf->pango_context);
+    tpf->pango_context = NULL;
+  }
+#endif
+  
+  if (tpf->typ == FONT_TYPE_TTF)
+  {
+    TTF_CloseFont(tpf->ttf_font);
+    tpf->ttf_font = NULL;
+  }
+  free(tpf);
+}
+
+TuxPaint_Font * TuxPaint_Font_OpenFont(const char * pangodesc, const char * ttffilename, int size)
+{
+  TuxPaint_Font * tpf = NULL;
+#ifndef NO_SDLPANGO
+  char desc[1024];
+#endif
+
+#ifdef DEBUG
+  printf("OpenFont(pango:\"%s\", ttf:\"%s\")\n", pangodesc, ttffilename);
+#endif
+
+#ifndef NO_SDLPANGO
+
+  if (pangodesc != NULL && pangodesc[0] != '\0')
+  {
+    tpf = (TuxPaint_Font *) malloc(sizeof(TuxPaint_Font));
+    tpf->typ = FONT_TYPE_PANGO;
+    snprintf(desc, sizeof(desc), "%s %d", pangodesc, (size * 3) / 4);
+
+#ifdef DEBUG
+    printf("Creating context: \"%s\"\n", desc);
+#endif
+
+    tpf->pango_context = SDLPango_CreateContext_GivenFontDesc(desc);
+    if (tpf->pango_context == NULL)
+    {
+#ifdef DEBUG
+      printf("Failed to load %s\n", desc);
+#endif
+      free(tpf);
+      tpf = NULL;
+    }
+    else
+      tpf->height = size; /* FIXME: Is this accurate!? -bjk 2007.07.12 */
+
+#ifdef DEBUG
+    printf("TuxPaint_Font_OpenFont() done\n"); fflush(stdout);
+#endif
+
+    return(tpf);
+  }
+#endif
+
+  if (ttffilename != NULL && ttffilename[0] != '\0')
+  {
+    int i;
+#ifdef DEBUG
+    printf("Opening TTF\n"); fflush(stdout);
+#endif
+      
+#ifdef __APPLE__
+    i = 0;
+    while (problemFonts[i] != NULL)
+    {
+      if (!strcmp(ttffilename, problemFonts[i++]))
+        return NULL;  /* bail on known problematic fonts that cause TTF_OpenFont to crash */
+    }
+#endif
+
+    tpf = (TuxPaint_Font *) malloc(sizeof(TuxPaint_Font));
+    tpf->typ = FONT_TYPE_TTF;
+    tpf->ttf_font = TTF_OpenFont(ttffilename, size);
+    if (tpf->ttf_font == NULL)
+    {
+#ifdef DEBUG
+      printf("Failed to load %s: %s\n", ttffilename, SDL_GetError());
+#endif
+      free(tpf);
+      tpf = NULL;
+    }
+    else
+    {
+#ifdef DEBUG
+      printf("Succeeded loading %s\n", ttffilename);
+#endif
+      tpf->height = TTF_FontHeight(tpf->ttf_font);
+    }
+  }
+
+#ifdef DEBUG
+  printf("TuxPaint_Font_OpenFont() done\n"); fflush(stdout);
+#endif
+
+  return(tpf);
+}
+
 
 #ifdef FORKED_FONTS
 
@@ -394,12 +560,14 @@ void receive_some_font_info(SDL_Surface * screen)
     if (buf_size <= buf_fill * 9 / 8 + 128)
     {
       buf_size = buf_size * 5 / 4 + 256;
+
+      // FIXME: Valgrind says this leaks -bjk 2007.07.19
       buf = realloc(buf, buf_size);
     }
     rc = read(font_socket_fd, buf + buf_fill, buf_size - buf_fill);
 #ifdef DEBUG
     printf("read: fd=%d buf_fill=%u buf_size=%u rc=%ld\n", font_socket_fd,
-	   buf_fill, buf_size, rc);
+	   buf_fill, buf_size, (long int) rc);
 #endif
 
     if (rc == -1)
@@ -445,7 +613,10 @@ void receive_some_font_info(SDL_Surface * screen)
   printf("Got %u bytes with %u families.\n", buf_fill, num_font_families);
 #endif
   user_font_families = malloc(num_font_families * sizeof *user_font_families);
+
+  // FIXME: Valgrind says this malloc() is leaked -bjk 2007.07.19
   fip = malloc(num_font_families * sizeof **user_font_families);
+
   i = num_font_families;
   while (i--)
   {
@@ -527,11 +698,12 @@ int load_user_fonts(SDL_Surface * screen, void *vp)
 #endif
   }
 
-  homedirdir = get_fname("fonts");
+  homedirdir = get_fname("fonts", DIR_DATA);
   loadfonts(screen, homedirdir);
   free(homedirdir);
+
 #ifdef WIN32
-  homedirdir = get_fname("data/fonts");
+  homedirdir = get_fname("data/fonts", DIR_DATA);
   loadfonts(screen, homedirdir);
   free(homedirdir);
 #endif
@@ -1045,37 +1217,149 @@ void groupfonts(void)
 }
 
 
-TTF_Font *getfonthandle(int desire)
+TuxPaint_Font *getfonthandle(int desire)
 {
   int missing = 0;
   family_info *fi = user_font_families[desire];
-  char *name = fi->filename[text_state];
+  char *name;
   char *pathname;
+  char description[1024];
+
+#ifdef DEBUG
+  printf("\ngetfonthandle(%d)...\n", desire); fflush(stdout);
+#endif
+
+  if (fi == NULL)
+  {
+#ifdef DEBUG
+    printf("getfonthandle(%d) points to a NULL family\n", desire);
+    fflush(stdout);
+#endif
+    return NULL;
+  }
+
+  if (fi->filename != NULL)
+  {
+#ifdef DEBUG
+    printf("Setting 'name' to fi->filename[%d (0x%x)]\n",
+           (int) text_state,
+           (int) text_state);
+    fflush(stdout);
+#endif
+
+    name = fi->filename[text_state];
+
+#ifdef DEBUG
+    printf("Which is: %s\n", name); fflush(stdout);
+#endif
+  }
+  else
+  {
+#ifdef DBEUG
+    printf("fi->filename is NULL\n"); fflush(stdout);
+#endif
+
+    name = NULL;
+  }
 
   if (fi->handle)
+  {
+#ifdef DEBUG
+    printf("fi->handle was set (0x%x)\n", (int) fi->handle); fflush(stdout);
+#endif
     return fi->handle;
+  }
 
-  if (!name)
-  {
-    name = fi->filename[text_state ^ TTF_STYLE_ITALIC];
-    missing = text_state & TTF_STYLE_ITALIC;
-  }
-  if (!name)
-  {
-    name = fi->filename[text_state ^ TTF_STYLE_BOLD];
-    missing = text_state & TTF_STYLE_BOLD;
-  }
-  if (!name)
-  {
-    name = fi->filename[text_state ^ (TTF_STYLE_ITALIC | TTF_STYLE_BOLD)];
-    missing = text_state & (TTF_STYLE_ITALIC | TTF_STYLE_BOLD);
-  }
-  pathname = alloca(strlen(fi->directory) + 1 + strlen(name) + 1);
-  sprintf(pathname, "%s/%s", fi->directory, name);
+#ifdef DEBUG
+  printf("fi->handle was not yet set\n"); fflush(stdout);
+#endif
 
-  fi->handle = TTF_OpenFont(pathname, text_sizes[text_size]);
+
+/* FIXME: Doesn't make sense; fi->handle is NULL! -bjk 2007.07.17
+
+#ifndef NO_SDLPANGO
+
+  if (fi->handle->typ == FONT_TYPE_PANGO)
+  {
+    snprintf(description, sizeof(description), "%s%s%s", fi->family,
+      (text_state ^ TTF_STYLE_ITALIC ? " italic" : ""),
+      (text_state ^ TTF_STYLE_BOLD ? " bold" : ""));
+
+    pathname = (char *) "";
+
+#ifdef DEBUG
+    printf("getfonthandle(%d) asking SDL_Pango for %s\n", desire, description);
+#endif
+  }
+#endif
+*/
+
+/* FIXME: Doesn't make sense; fi->handle is NULL! -bjk 2007.07.17
+  if (fi->handle->typ == FONT_TYPE_TTF)
+*/
+  {
+    if (!name)
+    {
+      name = fi->filename[text_state ^ TTF_STYLE_ITALIC];
+      missing = text_state & TTF_STYLE_ITALIC;
+    }
+    if (!name)
+    {
+      name = fi->filename[text_state ^ TTF_STYLE_BOLD];
+      missing = text_state & TTF_STYLE_BOLD;
+    }
+    if (!name)
+    {
+      name = fi->filename[text_state ^ (TTF_STYLE_ITALIC | TTF_STYLE_BOLD)];
+      missing = text_state & (TTF_STYLE_ITALIC | TTF_STYLE_BOLD);
+    }
+    if (!name)
+    {
+#ifdef DEBUG
+      printf("name is still NULL\n"); fflush(stdout);
+#endif
+      return(NULL);
+    }
+
+    pathname = alloca(strlen(fi->directory) + 1 + strlen(name) + 1);
+    sprintf(pathname, "%s/%s", fi->directory, name);
+
+    strcpy(description, "");
+  }
+
+  fi->handle = TuxPaint_Font_OpenFont(description, pathname, text_sizes[text_size]);
   // if the font doesn't load, we die -- it did load OK before though
-  TTF_SetFontStyle(fi->handle, missing);
+
+  if (fi->handle == NULL)
+  {
+#ifdef DEBUG
+    printf("fi->handle is NULL!\n"); fflush(stdout);
+#endif
+    return(NULL);
+  }
+
+  if (fi->handle->typ == FONT_TYPE_TTF)
+  {
+    if (fi->handle->ttf_font == NULL)
+    {
+#ifdef DEBUG
+      printf("fi->handle->ttf_font is NULL!\n"); fflush(stdout);
+#endif
+      return(NULL);
+    }
+
+#ifdef DEBUG
+    printf("calling TTF_SetFontStyle(0x%x)\n", missing); fflush(stdout);
+#endif
+
+    TTF_SetFontStyle(fi->handle->ttf_font, missing);
+  }
+
+#ifndef NO_SDLPANGO
+  if (fi->handle->typ == FONT_TYPE_PANGO)
+    printf("It's a Pango context...\n");
+#endif
+
   return fi->handle;
 }
 
@@ -1156,9 +1440,12 @@ int surfcmp(const void *s1, const void *s2)
 }
 
 // check if the characters will render distinctly
-int charset_works(TTF_Font * font, const char *s)
+int charset_works(TuxPaint_Font * font, const char *s)
 {
   SDL_Color black = { 0, 0, 0, 0 };
+#ifndef NO_SDLPANGO
+  SDLPango_Matrix pango_color;
+#endif
   SDL_Surface **surfs = malloc(strlen(s) * sizeof surfs[0]);
   unsigned count = 0;
   int ret = 0;
@@ -1166,12 +1453,28 @@ int charset_works(TTF_Font * font, const char *s)
   {
     char c[8];
     unsigned offset = 0;
-    SDL_Surface *tmp_surf;
+    SDL_Surface *tmp_surf = NULL;
+
     do
       c[offset++] = *s++;
     while ((*s & 0xc0u) == 0x80u);	// assume safe input
     c[offset++] = '\0';
-    tmp_surf = TTF_RenderUTF8_Blended(font, c, black);
+
+#ifndef NO_SDLPANGO
+    if (font->typ == FONT_TYPE_PANGO)
+    {
+      sdl_color_to_pango_color(black, &pango_color);
+      SDLPango_SetDefaultColor(font->pango_context, &pango_color);
+      SDLPango_SetText(font->pango_context, c, -1);
+      tmp_surf = SDLPango_CreateSurfaceDraw(font->pango_context);
+    }
+#endif
+
+    if (font->typ == FONT_TYPE_TTF)
+    {
+      tmp_surf = TTF_RenderUTF8_Blended(font->ttf_font, c, black);
+    }
+
     if (!tmp_surf)
     {
 #if 0
@@ -1199,3 +1502,110 @@ out:
   free(surfs);
   return ret;
 }
+
+int TuxPaint_Font_FontHeight(TuxPaint_Font * tpf)
+{
+  if (tpf == NULL)
+  {
+#ifdef DEBUG
+    printf("TuxPaint_Font_FontHeight() received NULL\n"); fflush(stdout);
+#endif
+    return(1);
+  }
+
+  return(tpf->height);
+}
+
+const char * TuxPaint_Font_FontFaceFamilyName(TuxPaint_Font * tpf)
+{
+  if (tpf == NULL)
+  {
+#ifdef DEBUG
+    printf("TuxPaint_Font_FontFaceFamilyName() received NULL\n"); fflush(stdout);
+#endif
+    return("");
+  }
+
+#ifndef NO_SDLPANGO
+  if (tpf->typ == FONT_TYPE_PANGO)
+  {
+    (void)(tpf);
+    /* FIXME */
+  
+    return("");
+  }
+#endif
+
+  if (tpf->typ == FONT_TYPE_TTF)
+    return (TTF_FontFaceFamilyName(tpf->ttf_font));
+
+#ifdef DEBUG
+  printf("TuxPaint_Font_FontFaceFamilyName() is confused\n");
+#endif
+
+  return("");
+}
+
+const char * TuxPaint_Font_FontFaceStyleName(TuxPaint_Font * tpf)
+{
+  if (tpf == NULL)
+  {
+#ifdef DEBUG
+    printf("TuxPaint_Font_FontFaceStyleName() received NULL\n"); fflush(stdout);
+#endif
+    return("");
+  }
+
+#ifndef NO_SDLPANGO
+  if (tpf->typ == FONT_TYPE_PANGO)
+  {
+    (void)(tpf);
+    /* FIXME */
+  
+    return("");
+  }
+#endif
+
+  if (tpf->typ == FONT_TYPE_TTF)
+    return (TTF_FontFaceStyleName(tpf->ttf_font));
+  
+#ifdef DEBUG
+  printf("TuxPaint_Font_FontFaceStyleName() is confused\n");
+#endif
+
+  return("");
+}
+
+
+#ifndef NO_SDLPANGO
+
+void sdl_color_to_pango_color(SDL_Color sdl_color,
+                              SDLPango_Matrix * pango_color)
+{
+  Uint8 pc[4][4];
+
+  pc[0][0] = 0;
+  pc[1][0] = 0;
+  pc[2][0] = 0;
+  pc[3][0] = 0;
+
+  pc[0][1] = sdl_color.r;
+  pc[1][1] = sdl_color.g;
+  pc[2][1] = sdl_color.b;
+  pc[3][1] = 255;
+
+  pc[0][2] = 0;
+  pc[1][2] = 0;
+  pc[2][2] = 0;
+  pc[3][2] = 0;
+
+  pc[0][3] = 0;
+  pc[1][3] = 0;
+  pc[2][3] = 0;
+  pc[3][3] = 0;
+
+  memcpy(pango_color, pc, 16);
+}
+
+#endif
+
