@@ -4,7 +4,7 @@
   For Tux Paint
   Language-related functions
 
-  Copyright (c) 2002-2007 by Bill Kendrick and others
+  Copyright (c) 2002-2008 by Bill Kendrick and others
   bill@newbreedsoftware.com
   http://www.tuxpaint.org/
 
@@ -23,37 +23,27 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   (See COPYING.txt)
 
-  $Id: i18n.c,v 1.26 2007/06/08 20:21:11 wkendrick Exp $
+  $Id: i18n.c,v 1.50 2008/06/27 20:20:06 wkendrick Exp $
 
-  June 14, 2002 - May 6, 2007
+  June 14, 2002 - June 27, 2008
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <libintl.h>
+#include <locale.h>
 #include "i18n.h"
 #include "debug.h"
 
-
 #ifdef WIN32
 #include <sys/types.h>
 #endif
 
-
-/* (Trouble building this for 10.2.8 target; bjk & mf 2006.01.14) */
-#ifndef __APPLE_10_2_8__
+#ifdef __BEOS__
 #include <wchar.h>
 #else
-#undef FORKED_FONTS
-#define wchar_t char
-#define wcslen strlen
-#define towupper toupper
-#define iswprint isprint
-#define OLD_UPPERCASE_CODE
-#endif
-
-#ifndef OLD_UPPERCASE_CODE
+#include <wchar.h>
 #include <wctype.h>
 #endif
 
@@ -65,6 +55,8 @@ static int langint = LANG_EN;
 const char *lang_prefixes[NUM_LANGS] = {
   "af",
   "ar",
+  "ast",
+  "az",
   "be",
   "bg",
   "bo",
@@ -76,10 +68,13 @@ const char *lang_prefixes[NUM_LANGS] = {
   "de",
   "el",
   "en",
-  "en_gb",
-  "en_za",
+  "en_AU",
+  "en_CA",
+  "en_GB",
+  "en_ZA",
+  "eo",
   "es",
-  "es_mx",
+  "es_MX",
   "et",
   "eu",
   "fi",
@@ -100,18 +95,22 @@ const char *lang_prefixes[NUM_LANGS] = {
   "it",
   "ja",
   "ka",
+  "km",
   "ko",
   "ku",
   "lt",
   "lv",
+  "mk",
   "ms",
   "nb",
   "nl",
   "nn",
   "nr",
+  "oc",
+  "oj",
   "pl",
-  "pt_br",
-  "pt_pt",
+  "pt_BR",
+  "pt_PT",
   "ro",
   "ru",
   "rw",
@@ -131,9 +130,11 @@ const char *lang_prefixes[NUM_LANGS] = {
   "ve",
   "vi",
   "wa",
+  "wo",
   "xh",
-  "zh_cn",
-  "zh_tw"
+  "zh_CN",
+  "zh_TW",
+  "zw"
 };
 
 
@@ -160,16 +161,31 @@ int lang_use_right_to_left[] = {
   -1
 };
 
+int lang_use_right_to_left_word[] = {
+#ifdef NO_SDLPANGO
+  LANG_HE,
+#endif
+  -1
+};
+
+int lang_y_nudge[][2] = {
+  {LANG_KM, 4},
+  {-1, -1}
+};
+
 
 char *langstr;
 int need_own_font;
 int need_right_to_left;
+int need_right_to_left_word;
 const char *lang_prefix;
 
 const language_to_locale_struct language_to_locale_array[] = {
   {"english", "C"},
   {"american-english", "C"},
   {"arabic", "ar_SA.UTF-8"},
+  {"asturian", "ast_ES.UTF-8"},
+  {"azerbaijani", "az_AZ.UTF-8"},
   {"croatian", "hr_HR.UTF-8"},
   {"hrvatski", "hr_HR.UTF-8"},
   {"catalan", "ca_ES.UTF-8"},
@@ -189,7 +205,10 @@ const language_to_locale_struct language_to_locale_array[] = {
   {"gujarati", "gu_IN.UTF-8"},
   {"british-english", "en_GB.UTF-8"},
   {"british", "en_GB.UTF-8"},
+  {"australian-english", "en_AU.UTF-8"},
+  {"canadian-english", "en_CA.UTF-8"},
   {"southafrican-english", "en_ZA.UTF-8"},
+  {"esperanto", "eo.UTF-8"},
   {"spanish", "es_ES.UTF-8"},
   {"mexican", "es_MX.UTF-8"},
   {"mexican-spanish", "es_MX.UTF-8"},
@@ -243,6 +262,7 @@ const language_to_locale_struct language_to_locale_array[] = {
   {"lithuanian", "lt_LT.UTF-8"},
   {"lietuviu", "lt_LT.UTF-8"},
   {"latvian", "lv_LV.UTF-8"},
+  {"macedonian", "mk_MK.UTF-8"},
   {"malay", "ms_MY.UTF-8"},
   {"dutch", "nl_NL.UTF-8"},
   {"nederlands", "nl_NL.UTF-8"},
@@ -250,6 +270,9 @@ const language_to_locale_struct language_to_locale_array[] = {
   {"nynorsk", "nn_NO.UTF-8"},
   {"norsk", "nn_NO.UTF-8"},
   {"ndebele", "nr_ZA.UTF-8"},
+  {"occitan", "oc_FR.UTF-8"},
+  {"ojibwe", "oj_CA.UTF-8"}, // Proper spelling
+  {"ojibway", "oj_CA.UTF-8"}, // For compatibility
   {"polish", "pl_PL.UTF-8"},
   {"polski", "pl_PL.UTF-8"},
   {"brazilian-portuguese", "pt_BR.UTF-8"},
@@ -275,10 +298,13 @@ const language_to_locale_struct language_to_locale_array[] = {
   {"ukrainian", "uk_UA.UTF-8"},
   {"walloon", "wa_BE.UTF-8"},
   {"walon", "wa_BE.UTF-8"},
+  {"wolof", "wo_SN.UTF-8"},
   {"xhosa", "xh_ZA.UTF-8"},
   {"chinese", "zh_CN.UTF-8"},
   {"simplified-chinese", "zh_CN.UTF-8"},
   {"traditional-chinese", "zh_TW.UTF-8"},
+  {"zapoteco", "zw_MX.UTF-8"},
+  {"khmer", "km_KH.UTF-8"}
 };
 
 
@@ -316,14 +342,17 @@ void ctype_utf8(void)
 
 /* Determine the current language/locale, and set the language string: */
 
-void set_current_language(void)
+int set_current_language(void)
 {
   char *loc;
   int i, found;
+  int y_nudge;
+
+  y_nudge = 0;
 
   bindtextdomain("tuxpaint", LOCALEDIR);
   /* Old version of glibc does not have bind_textdomain_codeset() */
-#if defined __GLIBC__ && __GLIBC__ == 2 && __GLIBC_MINOR__ >=2 || __GLIBC__ > 2
+#if defined __GLIBC__ && __GLIBC__ == 2 && __GLIBC_MINOR__ >=2 || __GLIBC__ > 2 || __APPLE__
   bind_textdomain_codeset("tuxpaint", "UTF-8");
 #endif
   textdomain("tuxpaint");
@@ -334,11 +363,16 @@ void set_current_language(void)
 
 
 #ifndef WIN32
-  loc = setlocale(LC_MESSAGES, NULL);
+  loc = setlocale(LC_MESSAGES, NULL); // NULL: Used to direct setlocale() to query the current internationalised environment and return the name of the locale().
+
+  // FIXME: I'm getting back en_US.UTF-8 even after LC_ALL has been putenv()'d...?? -bjk 2008.02.19
+
   if (loc != NULL)
   {
     if (strstr(loc, "LC_MESSAGES") != NULL)
+    {
       loc = getenv("LANG");
+    }
   }
 #else
   bind_textdomain_codeset("tuxpaint", "UTF-8");
@@ -377,16 +411,33 @@ void set_current_language(void)
     }
   }
 
+
+  /* FIXME: These don't work because we have the wrong langint...!? -bjk 2008.02.19 */
+
   lang_prefix = lang_prefixes[langint];
   need_own_font = search_int_array(langint, lang_use_own_font);
   need_right_to_left = search_int_array(langint, lang_use_right_to_left);
+  need_right_to_left_word = search_int_array(langint, lang_use_right_to_left_word);
+
+  for (i = 0; lang_y_nudge[i][0] != -1; i++)
+  {
+    // printf("lang_y_nudge[%d][0] = %d\n", i, lang_y_nudge[i][0]);
+    if (lang_y_nudge[i][0] == langint)
+    {
+      y_nudge = lang_y_nudge[i][1];
+      printf("y_nudge = %d\n", y_nudge);
+    }
+  }
 
 #ifdef DEBUG
-  fprintf(stderr, "DEBUG: Language is %s (%d) %s\n",
-	  lang_prefix, langint, need_right_to_left ? "(RTL)" : "");
+  fprintf(stderr, "DEBUG: Language is %s (%d) %s/%s\n",
+	  lang_prefix, langint,
+	  need_right_to_left ? "(RTL)" : "",
+	  need_right_to_left_word ? "(RTL words)" : "");
   fflush(stderr);
 #endif
 
+  return(y_nudge);
 }
 
 
@@ -407,6 +458,9 @@ void show_lang_usage(FILE * f, const char *const prg)
 /* af */ "  afrikaans\n"
 /* sq */ "  albanian\n"
 /* ar */ "  arabic\n"
+/* ast */ "  asturian\n"
+/* en_AU */ "  australian-english\n"
+/* az */ "  azerbaijani\n"
 /* eu */ "  basque       euskara\n"
 /* be */ "  belarusian   bielaruskaja\n"
 /* nb */ "  bokmal\n"
@@ -415,6 +469,7 @@ void show_lang_usage(FILE * f, const char *const prg)
 /* br */ "  breton       brezhoneg\n"
 /* en_GB */ "  british      british-english\n"
 /* bg_BG */ "  bulgarian\n"
+/* en_CA */ "  canadian-english\n"
 /* ca */ "  catalan      catala\n"
 /* zh_CN */ "  chinese      simplified-chinese\n"
 /* zh_TW */ "               traditional-chinese\n"
@@ -422,6 +477,7 @@ void show_lang_usage(FILE * f, const char *const prg)
 /* cs */ "  czech        cesky\n"
 /* da */ "  danish       dansk\n"
 /* nl */ "  dutch        nederlands\n"
+/* eo */ "  esperanto\n"
 /* et */ "  estonian\n"
 /* fo */ "  faroese\n"
 /* fi */ "  finnish      suomi\n"
@@ -446,11 +502,14 @@ void show_lang_usage(FILE * f, const char *const prg)
 /* ku */ "  kurdish\n"
 /* lv */ "  latvian\n"
 /* lt */ "  lithuanian   lietuviu\n"
+/* mk */ "  macedonian\n"
 /* ms */ "  malay\n"
 /* es_MX */
 	  "  mexican      mexican-spanish        espanol-mejicano\n"
 /* nr */ "  ndebele\n"
 /* nn */ "  norwegian    nynorsk                norsk\n"
+/* oc */ "  occitan\n"
+/* oj */ "  ojibwe       ojibway\n"
 /* pl */ "  polish       polski\n"
 /* pt_PT */ "  portuguese   portugues\n"
 /* ro */ "  romanian\n"
@@ -474,8 +533,10 @@ void show_lang_usage(FILE * f, const char *const prg)
 /* ve */ "  venda\n"
 /* vi */ "  vietnamese\n"
 /* wa */ "  walloon      walon\n"
+/* wo */ "  wolof\n"
 /* cy */ "  welsh        cymraeg\n"
 /* xh */ "  xhosa\n"
+/* zw */ "  zapoteco\n"
 	  "\n", prg);
 }
 
@@ -492,11 +553,15 @@ void show_locale_usage(FILE * f, const char *const prg)
 	  "  C       (English      American English)\n"
 	  "  af_ZA   (Afrikaans)\n"
 	  "  ar_SA   (Arabic)\n"
+	  "  ast_ES  (Asturian)\n"
+	  "  az_AZ   (Azerbaijani)\n"
 	  "  eu_ES   (Basque       Euskara)\n"
 	  "  be_BY   (Belarusian   Bielaruskaja)\n"
 	  "  nb_NO   (Bokmal)\n"
 	  "  pt_BR   (Brazilian    Brazilian Portuguese   Portugues Brazilian)\n"
 	  "  br_FR   (Breton       Brezhoneg)\n"
+	  "  en_AU   (Australian English)\n"
+	  "  en_CA   (Canadian English)\n"
 	  "  en_GB   (British      British English)\n"
 	  "  en_ZA   (South African English)\n"
 	  "  bg_BG   (Bulgarian)\n"
@@ -515,6 +580,7 @@ void show_locale_usage(FILE * f, const char *const prg)
 	  "  gos_NL  (Gronings     Zudelk Veenkelonioals)\n"
 	  "  gu_IN   (Gujarati)\n"
 	  "  de_DE   (German       Deutsch)\n"
+          "  eo      (Esperanto)\n"
 	  "  et_EE   (Estonian)\n"
 	  "  el_GR   (Greek)\n"
 	  "  he_IL   (Hebrew)\n"
@@ -532,8 +598,11 @@ void show_locale_usage(FILE * f, const char *const prg)
 	  "  ms_MY   (Malay)\n"
 	  "  lv_LV   (Latvian)\n"
 	  "  lt_LT   (Lithuanian   Lietuviu)\n"
+	  "  mk_MK   (Macedonian)\n"
           "  nr_ZA   (Ndebele)\n"
 	  "  nn_NO   (Norwegian    Nynorsk                Norsk)\n"
+	  "  oc_FR   (Occitan)\n"
+	  "  oj_CA   (Ojibway)\n"
 	  "  pl_PL   (Polish       Polski)\n"
 	  "  pt_PT   (Portuguese   Portugues)\n"
 	  "  ro_RO   (Romanian)\n"
@@ -558,12 +627,14 @@ void show_locale_usage(FILE * f, const char *const prg)
 	  "  ve_ZA   (Venda)\n"
 	  "  vi_VN   (Vietnamese)\n"
 	  "  wa_BE   (Walloon)\n"
+	  "  wo_SN   (Wolof)\n"
 	  "  cy_GB   (Welsh        Cymraeg)\n"
 	  "  xh_ZA   (Xhosa)\n"
+	  "  zw_MX   (Zapoteco)\n"
 	  "\n", prg);
 }
 
-void setup_language(const char *const prg)
+void setup_language(const char *const prg, int * y_nudge)
 {
 
 // Justify this or delete it. It seems to make Tux Paint
@@ -624,13 +695,14 @@ void setup_language(const char *const prg)
       putenv(s);
     }
 
-    setlocale(LC_ALL, "");
+    setlocale(LC_ALL, ""); // Specifies an implementation-dependent native environment. For XSI-conformant systems, this corresponds to the value of the associated environment variables, LC_* and LANG
+
     ctype_utf8();
     free(langstr);
     langstr = NULL;
   }
 
-  set_current_language();
+  *y_nudge = set_current_language();
 }
 
 
