@@ -4,13 +4,13 @@
   For Tux Paint
   Language-related functions
 
-  Copyright (c) 2002-2007 by Bill Kendrick and others
+  Copyright (c) 2002-2008 by Bill Kendrick and others
   bill@newbreedsoftware.com
   http://www.tuxpaint.org/
 
-  $Id: i18n.h,v 1.18 2007/04/26 20:13:47 wkendrick Exp $
+  $Id: i18n.h,v 1.29 2008/04/06 01:11:35 wkendrick Exp $
   
-  June 14, 2002 - April 26, 2007
+  June 14, 2002 - April 5, 2008
 */
 
 
@@ -26,6 +26,8 @@ enum
 {
   LANG_AF,			/* Afrikaans */
   LANG_AR,			/* Arabic */
+  LANG_AST,			/* Asturian */
+  LANG_AZ,			/* Azerbaijani */
   LANG_BE,			/* Belarusian */
   LANG_BG,			/* Bulgarian */
   LANG_BO,			/* Tibetan */
@@ -37,8 +39,11 @@ enum
   LANG_DE,			/* German */
   LANG_EL,			/* Greek */
   LANG_EN,			/* English (American) (DEFAULT) */
+  LANG_EN_AU,			/* English (Australian) */
+  LANG_EN_CA,			/* English (Canadian) */
   LANG_EN_GB,			/* English (British) */
   LANG_EN_ZA,			/* English (South African) */
+  LANG_EO,			/* Esperanto */
   LANG_ES,			/* Spanish */
   LANG_ES_MX,			/* Spanish (Mexican) */
   LANG_ET,			/* Estonian */
@@ -61,15 +66,19 @@ enum
   LANG_IT,			/* Italian */
   LANG_JA,			/* Japanese */
   LANG_KA,			/* Georgian */
+  LANG_KM,			/* Khmer */
   LANG_KO,			/* Korean */
   LANG_KU,			/* Kurdish */
   LANG_LT,			/* Lithuanian */
   LANG_LV,			/* Latvian */
+  LANG_MK,			/* Macedonian */
   LANG_MS,			/* Malay */
   LANG_NB,			/* Norwegian Bokmal */
-  LANG_NR,                      /* Ndebele */
   LANG_NL,			/* Dutch */
   LANG_NN,			/* Norwegian Nynorsk */
+  LANG_NR,                      /* Ndebele */
+  LANG_OC,			/* Occitan */
+  LANG_OJ,			/* Ojibway */
   LANG_PL,			/* Polish */
   LANG_PT_BR,			/* Portuguese (Brazilian) */
   LANG_PT_PT,			/* Portuguese (Portugal) */
@@ -92,9 +101,11 @@ enum
   LANG_VE,			/* Venda */
   LANG_VI,			/* Vietnamese */
   LANG_WA,			/* Walloon */
+  LANF_WO,                      /* Wolof */
   LANG_XH,			/* Xhosa */
   LANG_ZH_CN,			/* Chinese (Simplified) */
   LANG_ZH_TW,			/* Chinese (Traditional) */
+  LANG_ZW,			/* Zapoteco */
   NUM_LANGS
 };
 
@@ -115,7 +126,8 @@ extern int lang_use_own_font[];
 extern int lang_use_right_to_left[];
 extern char *langstr;
 extern int need_own_font;
-extern int need_right_to_left;
+extern int need_right_to_left; // Right-justify
+extern int need_right_to_left_word; // Words need to be reversed, too! (e.g., Hebrew, but not Arabic)
 extern const char *lang_prefix;
 extern const language_to_locale_struct language_to_locale_array[];
 
@@ -124,11 +136,11 @@ extern const language_to_locale_struct language_to_locale_array[];
 /* Function prototypes: */
 
 void set_langstr(const char *s);
-void set_current_language(void);
+int set_current_language(void);
 int get_current_language(void);
 void show_lang_usage(FILE * f, const char *const prg);
 void show_locale_usage(FILE * f, const char *const prg);
-void setup_language(const char *const prg);
+void setup_language(const char *const prg, int * y_nudge);
 void do_locale_option(const char *const arg);
 void ctype_utf8(void);
 
