@@ -4,12 +4,12 @@
   For Tux Paint
   List of available tools.
 
-  Copyright (c) 2002 by Bill Kendrick
+  Copyright (c) 2002-2007 by Bill Kendrick
   bill@newbreedsoftware.com
-  http://www.newbreedsoftware.com/tuxpaint/
+  http://www.tuxpaint.org/
 
-  June 14, 2002 - December 10, 2002
-  $Id: tools.h,v 1.6 2006/08/27 21:00:55 wkendrick Exp $
+  June 14, 2002 - July 17, 2007
+  $Id: tools.h,v 1.8 2007/07/26 16:30:36 wkendrick Exp $
 */
 
 
@@ -41,19 +41,46 @@ enum
 /* Tool names: */
 
 const char *const tool_names[NUM_TOOLS] = {
+  // Freehand painting tool
   gettext_noop("Paint"),
+
+  // Stamp tool (aka Rubber Stamps)
   gettext_noop("Stamp"),
+
+  // Line drawing tool
   gettext_noop("Lines"),
+
+  // Shape creation tool (square, circle, etc.)
   gettext_noop("Shapes"),
+
+  // Text tool
   gettext_noop("Text"),
+
+  // "Magic" effects tools (blur, flip image, etc.)
   gettext_noop("Magic"),
+
+  // Undo last action
   gettext_noop("Undo"),
+
+  // Redo undone action
   gettext_noop("Redo"),
+
+  // Eraser tool
   gettext_noop("Eraser"),
+
+  // Start a new picture
   gettext_noop("New"),
+
+  // Open a saved picture
   gettext_noop("Open"),
+
+  // Save the current picture
   gettext_noop("Save"),
+
+  // Print the current picture
   gettext_noop("Print"),
+
+  // Quit/exit Tux Paint application
   gettext_noop("Quit")
 };
 
@@ -61,27 +88,59 @@ const char *const tool_names[NUM_TOOLS] = {
 /* Some text to write when each tool is selected: */
 
 const char *const tool_tips[NUM_TOOLS] = {
+  // Paint tool instructions
   gettext_noop("Pick a color and a brush shape to draw with."),
+
+  // Stamp tool instructions
   gettext_noop("Pick a picture to stamp around your drawing."),
+
+  // Line tool instructions
   gettext_noop("Click to start drawing a line. Let go to complete it."),
-  gettext_noop
-    ("Pick a shape. Click to pick the center, drag, then let go when it is the size you want. Move around to rotate it, and click to draw it."),
-  gettext_noop
-    ("Choose a style of text. Click on your drawing and you can start typing."),
+
+  // Shape tool instructions
+  gettext_noop("Pick a shape. Click to pick the center, drag, then let go when it is the size you want. Move around to rotate it, and click to draw it."),
+
+  // Text tool instructions
+  gettext_noop("Choose a style of text. Click on your drawing and you can start typing."),
+
+  // Magic tool instruction
   gettext_noop("Pick a magical effect to use on your drawing!"),
-  /* Undo */ gettext_noop("Undo!"),
-  /* Redo */ gettext_noop("Redo!"),
-  /* Eraser */ gettext_noop("Eraser!"),
-  /* New */ gettext_noop("You now have a blank sheet to draw on!"),
-  /* Open */ gettext_noop("Open…"),
-  /* Save */ gettext_noop("Your image has been saved!"),
-  /* Print */ gettext_noop("Printing…"),
-  /* Quit */ gettext_noop("Bye bye!")
+
+  // Response to 'undo' action
+  gettext_noop("Undo!"),
+
+  // Response to 'redo' action
+  gettext_noop("Redo!"),
+
+  // Eraser tool
+  gettext_noop("Eraser!"),
+
+  // Response to 'start a new image' action
+  gettext_noop("Pick a color or picture with which to start a new drawing."),
+
+  // Response to 'open' action (while file dialog is being constructed)
+  gettext_noop("Open…"),
+
+  // Response to 'save' action
+  gettext_noop("Your image has been saved!"),
+
+  // Response to 'print' action (while printing, or print dialog is being used)
+  gettext_noop("Printing…"),
+
+  // Response to 'quit' (exit) action
+  gettext_noop("Bye bye!")
 };
 
+// Instruction while using Line tool (after click, before release)
 #define TIP_LINE_START gettext_noop("Let go of the button to complete the line.")
+
+// Instruction while using Shape tool (after first click, before release)
 #define TIP_SHAPE_START gettext_noop("Hold the button to stretch the shape.")
+
+// Instruction while finishing Shape tool (after release, during rotation step before second click)
 #define TIP_SHAPE_NEXT gettext_noop("Move the mouse to rotate the shape. Click to draw it.")
+
+// Notification that 'New' action was aborted (current image would have been lost)
 #define TIP_NEW_ABORT gettext_noop("OK then… Let’s keep drawing this one!")
 
 
